#
#  Copyright 1996-2013 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::fs - Filesystem type control module
#
package Security::fs;

use Security::util::die;
use Security::util::read_config;
use Sys::Filesystem;
use Exporter;
use strict;
use warnings "all";

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init );

sub init {
	print ( STDERR  "In Security::fs::init()\n") if ( $main::opts{'d'} );

	&read_config('ignorefs');

	# No longer need to register a wanted subroutine, since we are just
	# going to add the filesystems of our excluded types to the ignoredir
	# configuration option (see Security::find.pm).

	# if ignoredir is not defined, assume we have not yet called
	# Security::find::init(), and fail, otherwise it will overwrite
	# our adding directories to ignoredir
	&security_die ("ignoredir not set in config file or Security::fs::init() called before Security::find::init()\nPlease fix ignoredir or module load ordering\n")
		if ( ! exists $main::config{'ignoredir'} || ! scalar @{$main::config{'ignoredir'}} );

	my $fs = new Sys::Filesystem;
	my @filesystems = $fs->filesystems();
	for (@filesystems) {
		foreach my $i ( @{$main::config{'ignorefs'}} ) {
			if ( $fs->format($_) eq $i ) {
				my $dir = $_;
				$dir =~ s/\\040/ /g if ( $dir =~ m/\\040/ );
				push (@{$main::config{'ignoredir'}}, $dir);
				print STDERR "\tAdding $dir to ignoredir because it is of fstyp ($i)\n" 
					if ( $main::opts{'d'} );
			}
		}
	}
}

1;

__END__

=head1 NAME

Security::fs - File system type module

=head1 SYNOPSIS

Security::fs::check()

=head1 DESCRIPTION

Will prune filesystems based on a list of filesystem types not to
traverse.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Checks to see if L<Security::find::init()> has been called already,
so any values put into C<ignoredir> do not get lost.

Calls C<read_config (ignorefs)> to get the list of filesystem types
to prune out of our filesystem traversal.

Calls L<Sys::Filesystem::filesystems> to get the list of filesystems
that are currently mounted.  Matches up the filesystem type with the
list of filesystems to ignore in C<ignorefs>.  If a there is
a match, it will put the mountpoint on the C<ignoredir> list.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorefs>

The list of filesystem types to ignore during the filesystem traversal.

=back

=head1 SEE ALSO

L<Sys::Filesystem>

=cut

