#
#  Copyright 1996-2013 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::util::users - Cached getpwent data
#
package Security::util::users;

use User::pwent;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init check get );

our ($uids, $users);

sub init {
	my ($domain) = `/bin/domainname`;
	chomp $domain;

	# determine if we are running YP
	if ( ! exists $main::config{'yp'} ) {
		$main::config{'yp'} = `/bin/domainname 2>&1`;
		chomp $main::config{'yp'};
	}
	
	setpwent;
	while (my $pw = getpwent) {
		# Skip the YP map entry...
		if ( defined $pw ) {
			if ( $pw->name ne '+' ) {
				push @{$users->{$pw->name}}, $pw;
				push @{$uids->{$pw->uid}}, $pw;

			# If we are not running YP, then flag the '+' as an error
			} elsif ( ! exists $main::config{'yp'} || ! defined $main::config{'yp'} || ! length $$main::config{'yp'} ) {
				$Security::passwd::badlogin{$pw->name} = 1;
			}
		}
	}
	endpwent;
}

sub _byuid {
	if ( ref $users->{$a} eq 'ARRAY' && ref $users->{$b} eq 'ARRAY' ) {
		$users->{$a}->[0]->uid <=> $users->{$b}->[0]->uid;
	} else {
		return 1 if ( ref $users->{$a} eq 'ARRAY' );
		return -1 if ( ref $users->{$b} eq 'ARRAY' );
	}
}

sub get {
	my ($field, $value, $subfield) = @_;
	my (@res, $i);

	if ( defined $value && defined $subfield ) {
		if ( $field eq 'uid' && exists $uids->{$value} ) {
			if ( scalar @{$uids->{$value}} > 1 ) {
				for ( $i = 0; $i < scalar @{$uids->{$value}}; $i++ ) {
					push @res, $uids->{$value}->[$i]->$subfield;
				}
				return @res;
			} elsif ( scalar @{$uids->{$value}} == 1 ) {
				return $uids->{$value}->[0]->$subfield;
			} else {
				return undef;
			}
		} elsif ( exists $users->{$value} ) {
			if ( scalar @{$users->{$value}} > 1 ) {
				for ( $i = 0; $i < scalar @{$users->{$value}}; $i++ ) {
					push @res, $users->{$value}->[$i]->$subfield;
				}
				return @res;
			} elsif ( scalar @{$users->{$value}} == 1 ) {
				return $users->{$value}->[0]->$subfield;
			} else {
				return undef;
			}
		} else {
			return undef;
		}
	} else {
		return $field eq 'uid' 
			? sort keys %{$uids}
			: sort keys %{$users};
	}
}

sub check {
	my ($field, $value) = @_;
	my ($res);

	if ( $field eq 'uid' ) {
		$res = exists $uids->{$value};
	} else {
		$res = exists $users->{$value};
	}

	return $res;
}

1;

__END__

=head1 NAME

Security::util::users - module for checking against the password file.

=head1 SYNOPSIS

Security::util::users::check($field, $value)

=head1 DESCRIPTION

A wrapper routine around C<User::pwent::getpwent> calls to return the
password file information.

=head1 API

The API calls provided by this module

=over 4

=item C<init ()>

Reads in all the password information.

=item C<get ($field [, $value, $subfield])>

If I<$field> is the only argument, C<get()> will return a list all the
values, in uid order.  I<$field> can be either C<name> or C<uid>.  If
I<$value> and I<$subfield> are defined, C<get()> will treat $value as
either a login name or uid and return the specific $subfield value from
the password file.  

=item C<check ($field, $value)>

Returns true if I<$field> has I<$value>.  I<$field> can be either C<name>
or C<uid>.

=back

=head1 CONFIGURATION OPTIONS

None.

=head1 SEE ALSO

L<User::pwent>

L<perl>

=cut
