#!##PERL## -T
#
#  Copyright 1996-2013 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  security.pl - System Security Script
#

use Getopt::Std;
use File::Basename;
use POSIX;
use Net::SMTP;
use Security::util::die;
use Security::util::groups;
use Security::util::hostname;
use Security::util::print;
use Security::util::read_config;
use Security::util::SHA1;
use Security::util::users;
use strict;
use warnings "all";
no warnings 'once';

our $VERSION = '3.5.2';
our ($title, %config, %opts);
our $OWNER = 0;
our $GROUP = 0;
our $host = &security_hostname();
$opts{'C'} = '##SITEPREFIX##/etc/security/security.cf';
our $checksum_file = '##SITEPREFIX##/var/db/security/checksum.db';

sub usage {
	my ($opts) = @_;
	my $PROGNAME = File::Basename::basename($0);

	print "Usage: $PROGNAME -[$opts]\n";
	print "\t-C <conf file>\tUse <conf file> instead of default config file.\n";
	print "\t-d\t\tDebug mode\n";
	print "\t-h\t\tThis help screen\n";
	print "\t-M <addr>\tMail report to <addr>, defaults to 'root\@$host'\n";
	print "\t\t\tif <addr> starts with a /, write to a file.\n";
	print "\t-V\t\tPrint the program version ($VERSION)\n";
	exit 0;
}

sub init {
	print ( STDERR "In init\n") if ( $opts{'d'} );

	&report_header();

	# Check to make sure we are running as root
	if ( $< ne $OWNER ) {
		# If not, error but we can't read the mailhost var setting.
		$opts{'M'} = '-';
		&security_die("$0 not running as uid ($OWNER)\n");
	}
	
	# Get the checksum format
	&Security::util::SHA1::init();

	# Load up the password file information.
	&Security::util::users::init();

	# Load up the group file information.
	&Security::util::groups::init();

	&check_conf_files();
	&read_config('load');

	# Get the OS specific module loaded too
	# Put at the beginning of the list so we can
	# set vars used in other modules (i.e. exports.pm)
	unshift @{$config{'load'}}, "OS::$^O";

	&load_modules();
}

sub check_conf_files {
	print ( STDERR "In check_conf_files\n") if ( $opts{'d'} );

	# Now check if the configfile is ok.
	&check_conf_file ($opts{'C'}, 00400, $OWNER, $GROUP, 'config file');

	# Check if this script itself is ok
	&check_conf_file ($0, 00500, $OWNER, $GROUP, 'security script');

	# Check if the checksum db is ok
	&check_conf_file ($checksum_file, 00600, $OWNER, $GROUP, 'checksum db');
}

sub load_modules {
	print ( STDERR "In load_modules\n") if ( $opts{'d'} );

	# Now load all the requested modules from the config file.
	map {
		print STDERR "\tLoading Security::$_\n" if ($opts{'d'});
		my ($pkg) = "use Security::$_";
		eval $pkg;
		&security_die("Error loading module Security::$_ => $@\n") if ( $@ );
	} @{$config{'load'}};
}

sub report_header {
	my ($date);

	# Mail report information
	$date = strftime ( "%m/%d/%Y", localtime(time()));
	$title = "$host security ($VERSION) report for $date";
}

sub parse_options {
	my $opts = 'C:dhM:V';

	&usage($opts)
		if ( ! &getopts($opts, \%opts) || $opts{'h'} );

	if ( $opts{'V'} ) {
		print $VERSION . "\n";
		exit 0;
	}
}

sub call {
	my ($func) = @_;
	my ($cmd);
	print STDERR "In call($func)\n" if ( $opts{'d'} );

	foreach my $i ( @{$config{'load'}} ) {
		$cmd = "Security::${i}::${func}";
		no strict;
		&{$cmd} if ( defined &{$cmd} );
		use strict;
	}
}

sub check_conf_file {
	my ($file, $wanted_mode, $wanted_uid, $wanted_gid, $msg) = @_;
	print ( STDERR "In check_conf_file($file, ...)\n") if ( $opts{'d'} );
	my ($cursum);

    	my($dev,$ino,$mode,$nlink,$uid,$gid) = lstat($file);

	&security_die("Could not find file $file\n") 
		if ( ! ( defined $dev && defined $ino && defined $mode && defined $nlink && defined $uid && defined $gid )); 

	print STDERR "\t$file mode = " . sprintf("%o", $mode&07777) . ", uid = $uid, gid = $gid\n"
		if ( $opts{'d'} );

	&security_die("Permissions, owner or group not correct on $msg\n") 
		if (($mode & 07777) != $wanted_mode || 
			$uid != $wanted_uid || 
			$gid != $GROUP );
}

# Main.
&parse_options();
&init();
&call('init');
&call('check');
&call('print');
&call('destroy');
&print_close();

exit(0);

__END__

=head1 NAME

security.pl - File system security and verification program

=head1 SYNOPSIS

security.pl -[C:dhM:]

=head1 DESCRIPTION

A perl script to check for inconsistant, incorrect configurations that may
be a sign of a security issue.  Checks for stuff like files assigned a UID
that has no associated password entry, multiple UID 0 accounts, checksums
changing on set UID/GID files, etc.  Results are mailed to root or written
to a file on the system.

The script supports the following command line switches:

=over 4

=item I<-C conf file>

Use I<conf file> instead of default config file.

=item I<-d>

Display debugging trace output on STDERR.

=item I<-h>

Display all the command line switches the program supports.

=item I<-M email address | absolute file path>

Mail problem report to I<email address>.  If I<absoulte file path> is given,
write output to that file.  Defaults to mailing to I<root@$HOSTNAME>

=item I<-V>

Print version number and exit.

=back

=head1 API

The security program will support the following API calls for each module. 

=over 4

=item I<init()>

A routine to initalize any variables that will be needed later on.  Good time
to call I<read_config> (see below).  All of the OS specific modules will set
the default locations for system commands in this routine.

=item I<check()>

A routine to run our check of the system for any problems.  Storing results
in a module local variable to be displayed later.

=item I<print()>

A routine to take the values stored in the module local variable and format
the results.  Good time to call I<print_data>.

=item I<destroy()>

A routine to clean up any variables or commands that we might want to save
for a future run.

=back

the module.

=over 4

=item I<%main::opts>

The list of command line options stored in a hash.  Most common usage is
I<$main::opts{'d'}>, the debug flag.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<load>

The list of modules to dynamically load.  The script will automatically add
C<$^O> to the end of list.

=item C<mailhost>

This variable is SMTP host the mail should be sent to.  Defaults to
C<localhost>, but may not work on hosts that does not have a STMP agent
installed or running.

=back

=head1 HISTORY

This security script came out of enhancements to the C<FreeBSD> F</etc/security>
script which was in version 2.1.  After reading the C<HP-UX System Security
Manual> I felt there needed to be more checks.  After being rebuffed with 
some my suggestions, I decided to write and maintain my own script.

=head1 FILES

=over 4

=item security.cf

The configuration file for this script.

=item checksum.db

The list of SHA1 checksums for all files we are monitoring.  Stored in a
DBM file format.

=item Security::*.pm

All the plugin modules.

=back

=head1 SEE ALSO

L<Security::*>

L<aide(1)>

L<cisecurity.org>

L<tiger(8)>

L<tripwire(8)>

L<perl(1)>

L<FreeBSD PR 1538>

=cut
