#!/usr/bin/env perl
#
#  Copyright 1996-2013 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  08.t - passwd entry testing.
#

use Test;
use strict;
use warnings "all";

plan tests => 30;

use User::pwent qw/:DEFAULT pw_has/;
use Security::util::users;
use Security::passwd;
use Data::Dumper;

our (%opts, %config);
#$opts{'d'} = 1;

# Setup defaults
$config{'yp'} = '';
$config{'ignoreuid'} = [ 'toor' ];
$config{'ignorepasswd'} = [ 'sync' ];
$config{'ignoreduppasswd'} = [ 'root', 'toor' ];
$config{'ignoredupuid'} = [ 'sync', 'sync2' ];
$config{'ignoreexpired'} = [ 'notexpire' ];
$config{'defaultpass'} = [ '0123456789abc' ];

# Setup the fake test data
$Security::util::users::users = {
	'root' => [ &User::pwent::_populate( 'root', 'abc0123456789', 0, 0, undef, '', 'Super User', '/', '/bin/sh', 0 ) ],
	'toor' => [ &User::pwent::_populate( 'toor', 'abc0123456789', 0, 0, undef, '', 'Super User', '/', '/bin/sh', 0 ) ],
	'badroot' => [ &User::pwent::_populate( 'badroot', '0123456789abc', 0, 0, undef, '', 'Super User', '/', '/bin/sh', 0 ) ],
	'sync' => [ &User::pwent::_populate( 'sync', '', 5, 0, undef, '', '', '/', '/bin/sync', 0 ) ],
	'sync2' => [ &User::pwent::_populate( 'sync2', '', 5, 0, undef, '', '', '/', '/bin/sync', 0 ) ],
	'duppass' => [ &User::pwent::_populate( 'duppass', 'a0b1c2d3e4f5g6h', 6, 0, undef, '', '', '/', '/bin/false', 0 ) ],
	'duppass2' => [ &User::pwent::_populate( 'duppass2', 'a0b1c2d3e4f5g6h', 8, 0, undef, '', '', '/', '/bin/false', 0 ) ],
	'baddup' => [ &User::pwent::_populate( 'baddup', '*', 7, 0, undef, '', '', '/', '/bin/false', 0 ) ],
	'baddup2' => [ &User::pwent::_populate( 'baddup2', '!!', 7, 0, undef, '', '', '/', '/bin/false', 0 ) ],
	'bad,char' => [ &User::pwent::_populate( 'bad,char', 'x', 9, 10, undef, '', '', '/', '/bin/false', 0 ) ],
	'reallyreallyreallyreallylongloginname' => [ &User::pwent::_populate( 'reallyreallyreallyreallylongloginname', '*LOCKED*', 10, 10, undef, '', '', '/', '/bin/false', 0 ) ],
	'baduid' => [ &User::pwent::_populate( 'baduid', 'x', -10, 10, undef, '', '', '/', '/bin/false', 0 ) ],
	'badgid' => [ &User::pwent::_populate( 'badgid', 'NP', 11, 4300000000, undef, '', '', '/', '/bin/false', 0 ) ],
	'expire' => [ &User::pwent::_populate( 'expire', 'NP', 12, 10, undef, '', '', '/', '/bin/false', 1 ) ],
	'notexpire' => [ &User::pwent::_populate( 'notexpire', 'NP', 13, 10, undef, '', '', '/', '/bin/false', 1 ) ],
};

# Sync the uids hash with the users hash data
foreach my $name (keys %{$Security::util::users::users} ) {
	foreach my $pw ( @{$Security::util::users::users->{$name}} ) {
		push @{$Security::util::users::uids->{$pw->uid}}, $pw;
	}
}

sub print_data {}

&Security::passwd::init();
&Security::passwd::check();

# Make sure we found all the bad root accounts
ok ( scalar keys %Security::passwd::uid0, 1, "Did not find right number of bad uid 0 account");
ok ( $Security::passwd::uid0{'toor'}, undef, "Wrong bad uid 0 account found");
ok ( $Security::passwd::uid0{'badroot'}, 1, "Wrong bad uid 0 account found");

# Now check for no passwords
ok ( scalar keys %Security::passwd::nopass, 1, "Did not find right number of accounts w/ missing passwords");
ok ( $Security::passwd::nopass{'sync'}, undef, "Wrong missing password account found");
ok ( $Security::passwd::nopass{'sync2'}, 1, "Wrong missing password account found");

# Now check for duplicate passwords
ok ( scalar keys %Security::passwd::duppasswd, 2, "Did not find right number of accounts w/ duplicate passwords");
ok ( $Security::passwd::duppasswd{'toor'}, undef, "Duplicate password found for toor (and should not have been)");
ok ( $Security::passwd::duppasswd{'root'}, undef, "Duplicate password found for root (and should not have been)");
ok ( $Security::passwd::duppasswd{'sync'}, undef, "Duplicate password found for sync (and should not have been)");
ok ( $Security::passwd::duppasswd{'sync2'}, undef, "Duplicate password found for sync2 (and should not have been)");
ok ( $Security::passwd::duppasswd{'duppass'}, 'duppass and duppass2', "Duplicate password found for duppass2");
ok ( $Security::passwd::duppasswd{'duppass2'}, 'duppass and duppass2', "Duplicate password found for duppass2");

# Now check for default passwords
ok ( scalar keys %Security::passwd::defaultpw, 1, "Did not find right number of accounts w/ default password");
ok ( $Security::passwd::defaultpw{'badroot'}, 1, "Default password not found for badroot");

# Now check for duplicate uids
ok ( scalar keys %Security::passwd::dupuid, 2, "Did not find right number of accounts w/ duplicate uids");
ok ( $Security::passwd::dupuid{'baddup'}, 'baddup and baddup2', "Duplicate UID not found for baddup");
ok ( $Security::passwd::dupuid{'baddup2'}, 'baddup and baddup2', "Duplicate UID found for baddup2");
ok ( $Security::passwd::dupuid{'sync'}, undef, "Duplicate UID found for sync (and should not have been)");
ok ( $Security::passwd::dupuid{'sync2'}, undef, "Duplicate UID found for sync2 (and should not have been)");

# Now check for bad login info (names, uid, gid)
ok ( scalar keys %Security::passwd::badlogin, 2, "Did not find right number of accounts w/ bad login info");
ok ( $Security::passwd::badlogin{'reallyreallyreallyreallylongloginname'}, 1, "Did not find account with really long name");
ok ( $Security::passwd::badlogin{'bad,char'}, 1, "Did not find account with invalid charater");
ok ( scalar keys %Security::passwd::baduid, 1, "Did not find right number of accounts w/ bad login info");
ok ( $Security::passwd::baduid{'baduid'}, 1, "Did not find account with bad uid");
ok ( scalar keys %Security::passwd::badgid, 1, "Did not find right number of accounts w/ bad login info");
ok ( $Security::passwd::badgid{'badgid'}, 1, "Did not find account with bad gid");

# Now check for expired accounts
skip (
	( !pw_has('expire') ? "Expire not supported on this host": 0),
	scalar keys %Security::passwd::expired, 1,
);
skip (
	( !pw_has('expire') ? "Expire not supported on this host": 0),
	$Security::passwd::expired{'expire'}, 1,
);
skip (
	( !pw_has('expire') ? "Expire not supported on this host": 0),
	$Security::passwd::expired{'notexpire'}, undef,
);

exit 0;
