#!/usr/bin/env perl
#
#  Copyright 1996-2013 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  14.t - OS Specific module testing
#

use Test;
use strict;
use warnings "all";
use t::common;
use Security::pkgchk;
use Security::OS::freebsd;
use Security::OS::linux;

plan tests => 29;

# Some common defaults
our (%opts, %config);
#$opts{'d'} = 1;

my ($tmpfile) = 't/pkginfo.out';
$config{'ignorepkgfiles'} = [];
my ($content, $version);
$version = 'X.Y.Z';

$content = "
Information for security-$version:

Mismatched Checksums:
./sbin/security.pl fails the original MD5 checksum
./t/14.t fails the original MD5 checksum
pkg_info: ./t/security.cf doesn't exist
";
&create_file($tmpfile, $<, $(, 0640, $content);
$Security::OS::freebsd::pkg_info = "cat $tmpfile";
$config{'ignorepkgfiles'} = ['./t/14.t'];

&Security::OS::freebsd::check_pkg_info();

ok ( scalar keys %Security::OS::freebsd::pkgs, 2, 'Could not find the correct number of bad pacakges');
ok ( $Security::OS::freebsd::pkgs{'./sbin/security.pl'}, "checksum mismatch (package security-$version)", 'Could not find the checksum error');
ok ( $Security::OS::freebsd::pkgs{'./t/14.t'}, undef, 'Could not find the checksum error');
ok ( $Security::OS::freebsd::pkgs{'./t/security.cf'}, "file not found (package security-$version)", 'Could not find missing file error');

#&Security::OS::freebsd::check_osfiles();

ok ( scalar keys %Security::OS::freebsd::perms, 0, 'Could not find the correct number of mtree errors');

# Run pkgchk specific tests
$content = '
ERROR: ./sbin/security.pl
    modtime <08/06/10 12:32:24 AM> expected <04/14/11 10:25:37 AM> actual
    file size <40559> expected <40791> actual
    file cksum <3907> expected <20568> actual
    permissions <0555> expected <0755> actual
ERROR: ./sbin/generate_config.pl
    owner name <sys> expected <root> actual
    group name <sys> expected <root> actual
ERROR: ./t
    owner name <sys> expected <root> actual
    permissions <0555> expected <0755> actual
    file type <d> expected <l> actual
ERROR: ./t/14.t
    permissions <0555> expected <0755> actual
';
&create_file($tmpfile, $<, $(, 0640, $content);
$Security::pkgchk::pkgchk = "cat $tmpfile";

&Security::pkgchk::check();

ok ( scalar keys %Security::pkgchk::perms, 3, "Could not find the correct number of pkgchk errors");
ok ( $Security::pkgchk::perms{'./sbin/security.pl'}->{'modtime'}, '(08/06/10 12:32:24 AM:04/14/11 10:25:37 AM)', "Could not find the modtime error");
ok ( $Security::pkgchk::perms{'./sbin/security.pl'}->{'file size'}, '(40559:40791)', "Could not find the file size error");
ok ( $Security::pkgchk::perms{'./sbin/security.pl'}->{'file cksum'}, '(3907:20568)', "Could not find the file cksum error");
ok ( $Security::pkgchk::perms{'./sbin/security.pl'}->{'permissions'}, '(0555:0755)', "Could not find the permissions error");
ok ( $Security::pkgchk::perms{'./sbin/generate_config.pl'}->{'owner name'}, '(sys:root)', "Could not find the owner name error");
ok ( $Security::pkgchk::perms{'./sbin/generate_config.pl'}->{'group name'}, '(sys:root)', "Could not find the group name error");
ok ( $Security::pkgchk::perms{'./t'}->{'owner name'}, '(sys:root)', "Could not find the owner name error");
ok ( $Security::pkgchk::perms{'./t'}->{'permissions'}, '(0555:0755)', "Could not find the permissions error");
ok ( $Security::pkgchk::perms{'./t'}->{'file type'}, '(d:l)', "Could not find the file type error");
ok ( $Security::pkgchk::perms{'./t/14.t'}, undef, "Found an error we should have ignored");

# Run chkstat specific tests
$content = '
Checking permissions and ownerships - using the permissions files
	/etc/permissions
	/etc/permissions.secure
fscaps support disabled (file_caps missing in /proc/cmdline).
./sbin/security.pl should be root:trusted 4750. (wrong permissions 4755)
./t should be root:trusted 4750. (wrong permissions 4755)
./t/14.t should be root:root 0600. (wrong permissions 0644)
';
&create_file($tmpfile, $<, $(, 0640, $content);
$Security::OS::linux::chkstat = "cat $tmpfile";
&Security::OS::linux::check_permissions();

ok ( scalar keys %Security::OS::linux::perms2, 2, "Could not find the correct number of chkstat errors");
ok ( $Security::OS::linux::perms2{'./sbin/security.pl'}, 'should be root:trusted 4750. (wrong permissions 4755)', "Could not find the error");
ok ( $Security::OS::linux::perms2{'./t'}, 'should be root:trusted 4750. (wrong permissions 4755)', "Could not find the error");
ok ( $Security::OS::linux::perms2{'./t/14.t'}, undef, "Found an error we should have ignored");

# Run apt-key list specific tests
$content = '
/etc/apt/trusted.gpg
--------------------
pub   1024D/437D05B5 2004-09-12
uid                  Ubuntu Archive Automatic Signing Key <ftpmaster@ubuntu.com>
sub   2048g/79164387 2004-09-12

pub   1024D/FBB75451 2004-12-30
uid                  Ubuntu CD Image Automatic Signing Key <cdimage@ubuntu.com>

';
&create_file($tmpfile, $<, $(, 0640, $content);
$config{'gpgkeys'} = [ '437D05B5' ];
$Security::OS::linux::aptkey = "cat $tmpfile";
&Security::OS::linux::check_dpkg_signers();
ok ( scalar keys %Security::OS::linux::sigs, 1, "Could not find the correct number of apt-key errors");
ok ( $Security::OS::linux::sigs{'437D05B5'}, undef, "Could not find the bad gpg key");
ok ( $Security::OS::linux::sigs{'FBB75451'}, 'untrusted key loaded into GPG keyring', "Could not find the bad gpg key");

# Run debsums specific tests
$content = '
debsums: no md5sums for foo
debsums: no md5sums for bar
debsums: missing file ./TODO (from security package)
debsums: changed file ./t/14.t (from security package)
debsums: changed file ./t/common.pm (from security package)
';
&create_file($tmpfile, $<, $(, 0640, $content);
$Security::OS::linux::debsums = "cat $tmpfile";
$config{'ignorepkgfiles'} = ['./t/14.t', 'foo'];

&Security::OS::linux::check_dpkg_files();

ok ( scalar keys %Security::OS::linux::perms, 3, "Could not find the correct number of debsum errors");
ok ( $Security::OS::linux::perms{'foo'}, undef, "Should not have found this package md5sum error");
ok ( $Security::OS::linux::perms{'bar'}, 'package has no md5sums', "Did not find this package md5sum error");
ok ( $Security::OS::linux::perms{'./TODO'}, 'file is missing (package security)', "Did not find this missing file error");
ok ( $Security::OS::linux::perms{'./t/14.t'}, undef, "Should not have found this changed file error");
ok ( $Security::OS::linux::perms{'./t/common.pm'}, 'checksum mismatch (package security)', "Did not find this changed file error");

unlink (@t::common::FILES);

exit 0;
